#include "Camera.h"
#include <iostream>
#include "glm/gtx/string_cast.hpp"
#include "quaternion_utils.h"




Camera::Camera() : Camera(vec3(7.0f, 3.0f, 0.0f))
{

}

Camera::Camera(vec3 position) : Camera(position, vec3(0.0f, 0.0f, 0.0f))  {

}

Camera::Camera(vec3 position, vec3 lookAt)
{
	cameraMove = glm::vec3(0.0f, 0.0f, 0.0f);
	this->position = position;
	vec3 dir = normalize(lookAt -position);
	verticalAngle = asinf(dir.y);
	horizontalAngle = atan2(dir.x, dir.z);

	rotate(0, 0); //sets direction
	up = glm::vec3(0.0f, 1.0f, 0.0f);

}


Camera::~Camera()
{
}

void Camera::move(glm::vec3 relative)
{
	if (length(relative) < 0.001f) return;

	relative = normalize(relative);
	glm::vec3 right = cross(direction, up);
	vec3 uplocal = cross(right, direction);

	position += (relative.x*speed* direction)+ (relative.y*speed*uplocal)+ (relative.z*speed* right);
}

void Camera::rotate(int dx, int dy)
{
	horizontalAngle += mouseSpeed  * float(dx);
	verticalAngle += mouseSpeed  * float(dy);

	direction = glm::vec3(
		cos(verticalAngle) * sin(horizontalAngle),
		sin(verticalAngle),
		cos(verticalAngle) * cos(horizontalAngle)
	);
	
}

void Camera::applyMatrix()
{
	glm::vec3 p = position + direction;
	gluLookAt(position.x, position.y, position.z,
		p.x, p.y, p.z, up.x, up.y, up.z);
}

void Camera::mouseButton(int button, int state, int x, int y) {
	lastX = x;
	lastY = y;

}

void Camera::mouseMove(int x, int y) {
	rotate(lastX - x, lastY - y);
	lastX = x;
	lastY = y;

}

void Camera::keyPressed(unsigned char key, int x, int y)
{
	switch (key) {
	case 'w':
		cameraMove += (glm::vec3(1.0f, 0.0f, 0.0f));
		break;
	case 's':
		cameraMove += (glm::vec3(-1.0f, 0.0f, 0.0f));
		break;
	case 'a':
		cameraMove += (glm::vec3(0.0f, 0.0f, -1.0f));
		break;
	case 'd':
		cameraMove += (glm::vec3(0.0f, 0.0f, 1.0f));
		break;
	case 'q':
		cameraMove += (glm::vec3(0.0f, 1.0f, 0.0f));
		break;
	case 'e':
		cameraMove += (glm::vec3(0.0f, -1.0f, 0.0f));
		break;
	}
}


void  Camera::keyReleased(unsigned char key, int x, int y) {
	switch (key) {
	case 'w':
		cameraMove -= (glm::vec3(1.0f, 0.0f, 0.0f));
		break;
	case 's':
		cameraMove -= (glm::vec3(-1.0f, 0.0f, 0.0f));
		break;
	case 'a':
		cameraMove -= (glm::vec3(0.0f, 0.0f, -1.0f));
		break;
	case 'd':
		cameraMove -= (glm::vec3(0.0f, 0.0f, 1.0f));
		break;
	case 'q':
		cameraMove -= (glm::vec3(0.0f, 1.0f, 0.0f));
		break;
	case 'e':
		cameraMove -= (glm::vec3(0.0f, -1.0f, 0.0f));
		break;

	}


}

void Camera::tick()
{
	/*
	int timeSinceStart = glutGet(GLUT_ELAPSED_TIME);
	int deltaTime = timeSinceStart - oldTimeSinceStart;
	oldTimeSinceStart = timeSinceStart;
	*/
	move(cameraMove);
}
